#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <ext/hash_map>
#include <functional>
#include <utility>

using namespace std;
using namespace __gnu_cxx;

struct eqstr
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) == 0;
  }
};

typedef hash_multimap<const char*, int, hash<const char*>, eqstr> map_type;

vector< int > lookup(const map_type& Map, const char* str)
{
    vector< int > starts;
    
  pair<map_type::const_iterator, map_type::const_iterator> p =
    Map.equal_range(str);

  for (map_type::const_iterator i = p.first; i != p.second; ++i)
      starts.push_back( (*i).second);

  return starts;
  
}


vector< int > find_starts_of_tags( const string &genome, const string &tag)
{
    vector< int > starts;

    int pos = 0;

    bool done = false;

    for(int i = 0, tfind; (tfind = genome.find(tag, i)) != string::npos; i = tfind + 1)
    {
	starts.push_back( tfind );	
    }

    return starts;
}

char numtochar( int num )
{
    if ( num == 0 )
	return 'A';
    if ( num == 1 )
	return 'T';
    if ( num == 2 )
	return 'C';
    if ( num == 3 )
	return 'G';
}

main()
{
    // create a hashtable keyed by tag and storing the start locations in the genome
    map_type M;

    ifstream genome_file("dros_x2.fasta");
    
    string genome, temp;

    while( getline( genome_file, temp ))
	genome += temp;

    std::cout << genome.size() << "\n";
    
    vector< string > ditags;

    vector< string > left_ditags;
    vector< string > right_ditags;

    ifstream ditag_file("ditags.txt");

    string ditag, ltag, rtag;

    while( getline( ditag_file, ditag ))
    {
	ditags.push_back( ditag );
	left_ditags.push_back( ditag.substr(0,11) );
	right_ditags.push_back( ditag.substr(11,11) );
	
    }

    std::cout << ditags.size() << endl;
    std::cout << left_ditags.size() << endl;
    std::cout << right_ditags.size() << endl;

    
    // initialize with -1 t show it exists as a tag. We will then see if a tag exists
    //    before storing a string loc in the gnome.
    for ( int t = 0; t < ditags.size(); t++ )
    {
	if ( M.count( left_ditags[t].c_str()) == 0 )
	{
	    M.insert(map_type::value_type(left_ditags[t].c_str(), -1));
	}
	
	if ( M.count( right_ditags[t].c_str()) == 0 )
	    M.insert(map_type::value_type(right_ditags[t].c_str(), -1));
    }
    
    // now go through the genome making every possible 11 char string.
    // If the string is a hashed tag, then add the location to the bucket

    int len = 11;
    
    for (int pos = 0; pos < genome.size() - 11; pos++ )
    {
	if ( !(pos%1000000) )
	{
	    printf("pos %d\n",pos );
	}
	
	char *genome_tag = new char[11];
	bool has_N = false;
	
	for ( int i = 0; i < 11; i++ )
	{
	    genome_tag[i] = genome[pos + i];
	    if ( genome[pos+i] == 'N' )
	    {
		has_N = true;
		pos = pos + i;
	    }
	    
	}
	
	if ( !has_N && M.count( genome_tag ) > 0 ) // this means there must have been a ditag for this
	{
	    M.insert(map_type::value_type(genome_tag, pos));
	}
	else
	    delete [] genome_tag;
	
    }

    for ( int t = 0; t < ditags.size(); t++ )
    {
	vector< int > ltest = lookup( M, left_ditags[t].c_str() );
	vector< int > rtest = lookup( M, right_ditags[t].c_str() );
	cout << endl;

	printf("ditag %d %s: left #matches %d right #matches %d:\n",t,ditags[t].c_str(), ltest.size(), rtest.size());
	if ( ltest.size() > 1 && rtest.size() > 1 && ltest.size() < 5000 && rtest.size() < 5000 )
	{
	    // search for rights and lefts the correct distance apart
	    for ( int i = 0; i < ltest.size(); i++ )
	    {
		for ( int j = 0; j < rtest.size(); j++ )
		{
		    if ( rtest[j] != -1 && ltest[i] != -1 )
		    {
			int dist = rtest[j] - ltest[i];
			if ( dist > 14500 && dist < 20500 )
			    printf("\t left %d right %d dist %d\n",ltest[i], rtest[j],dist);
		    }
		}
	    }
	}
	else
	    printf("NO CHECK: size ltest %d rtest %d\n",ltest.size(), rtest.size() );
    }
}
